{% load i18n %}<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" {{ gmap.xmlns }}>
<head>
  <title>{% block title %}{% trans "Google Maps via GeoDjango" %}{% endblock %}</title>
  {{ gmap.style }}
  {{ gmap.scripts }}
</head>
<body {{ gmap.onload }} {{ gmap.onunload }}>
{% if gmap.dom_ids %}{% for dom_id in gmap.dom_ids %}<div id="{{ dom_id }}" style="width:600px;height:400px;"></div>{% endfor %}
{% else %}<div id="{{ gmap.dom_id }}" style="width:600px;height:400px;"></div>{% endif %}
</body>
</html>
